__title__ = 'slim.models.fields'
__version__ = '0.1'
__build__ = 0x000001
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__all__ = ('LanguageField', 'SimpleLanguageField')

from django.db import models
from django.core import exceptions
from django.utils.translation import ugettext_lazy as _

from slim import get_languages, default_language, get_languages_keys
from slim.monkey_patches import monkeypatch_method, monkeypatch_property
from slim.translations import is_primary_language


class LanguageField(models.CharField):
    """
    LanguageField model. Stores language string in a ``CharField`` field.

    Using `contrib_to_class` melthod adds `translation_of` field, which is simply a ``ForeignKey``
    to the same class.
    """
    __metaclass__ = models.SubfieldBase

    def __init__(self, *args, **kwargs):
        """
        Create new field. Argument ``populate`` will be sent as-is to the form field.
        """
        defaults = {
            'verbose_name': _('Language'),
            'populate': None,
            'max_length': 10,
            'choices': get_languages(),
            'default': default_language
        }
        defaults.update(kwargs)
        self.populate = defaults.pop('populate', None)
        super(LanguageField, self).__init__(*args, **defaults)

    def contribute_to_class(self, cls, name):
        """
        Language field consists of more than one database record. We have ``lanaguage`` (CharField)
        and ``translation_of`` (ForeignKey to ``cls``) in order to identify translated and
        primary objects.

        We have a set of very useful methods implemented in order to get translations easily.
        """
        self.name = name
        self.translation_of = models.ForeignKey(cls, blank=True, null=True, verbose_name=_('Translation of'), \
                                                related_name='translations', \
                                                limit_choices_to={'language': default_language}, \
                                                help_text=_('Leave this empty for entries in the primary language.'))
        cls.add_to_class('translation_of', self.translation_of)
        super(LanguageField, self).contribute_to_class(cls, name)

        @monkeypatch_property(cls)
        def is_multilingual(self):
            """
            Simple flat to use on objects to find our wheither they are multilinugal
            or not

            :return bool: Always returns boolean True
            """
            return True

        @monkeypatch_method(cls)
        def get_redirect_to_target(self, request):
            """
            Find an acceptable redirect target. If this is a local link, then try
            to find the page this redirect references and translate it according
            to the user's language. This way, one can easily implement a localized
            "/"-url to welcome page redirection.
            """
            target = self.redirect_to
            if target and target.find('//') == -1: # Not an offsite link http://bla/blubb
                try:
                    page = cls.objects.page_for_path(target)
                    page = page.get_translation(getattr(request, 'LANGUAGE_CODE', None))
                    target = page.get_absolute_url()
                except cls.DoesNotExist:
                    pass
            return target

        @monkeypatch_method(cls)
        def available_translations(self):
            """
            Returns available translations.

            :return interable: At this moment a list of objects.
            """
            if not self.id: # New, unsaved pages have no translations
                return []
            if is_primary_language(self.language):
                return self.translations.all()
            elif self.translation_of:
                return [self.translation_of] + list(self.translation_of.translations.exclude(
                    language=self.language))
            else:
                return []

        @monkeypatch_method(cls)
        def get_original_translation(self, *args, **kwargs):
            """
            Gets original translation of current object.

            :return obj: Object of the same class as the one queried.
            """
            if is_primary_language(self.language):
                return self
            return self.translation_of

        @monkeypatch_method(cls)
        def translation_admin(self, *args, **kwargs):
            """
            Gets a HTML with URL to the original translation of available. For admin use.

            :return str:
            """
            if self.translation_of:
                url = u'/admin/' + self._meta.app_label + '/' + self._meta.module_name + '/' + \
                      str(self.translation_of.id) + '/'
                name = unicode(self.translation_of)

                return u'<a href="%(url)s">%(name)s</a>' % {'url': url, 'name': name}
            return ''
        translation_admin.allow_tags = True
        translation_admin.short_description = _('Translation of')

        @monkeypatch_method(cls)
        def _available_translations_admin(self, include_self=True):
            """
            Gets a HTML with all available translation URLs for current object if available. For admin use.

            :return str:
            """
            try:
                original_translation = self.original_translation
                available_translations = list(self.available_translations())
                languages_keys = get_languages_keys()
                languages = dict(get_languages())

                if include_self:
                    available_translations.append(self)

                output = []
                # Processing all available translations. Adding edit links.
                if available_translations:
                    for translation in available_translations:
                        url = u'/admin/' + translation._meta.app_label + '/' + translation._meta.module_name + \
                              '/' + str(translation.id) + '/'
                        name = unicode(languages[translation.language])
                        output.append(u'<a href="%(url)s">%(name)s</a>' % {'url': url, 'name': name})
                        languages_keys.remove(translation.language)

                if self.pk and self.language in languages_keys:
                    languages_keys.remove(self.language)

                # For all languages that are still available (original object has no translations for)
                for language in languages_keys:
                    url = u'/admin/' + self._meta.app_label + '/' + self._meta.module_name + \
                          '/add/?translation_of=' + str(original_translation.id) + '&amp;language=' + language
                    name = unicode(languages[language])
                    output.append(u'<a href="%(url)s" style="color:#baa">%(name)s</a>' % {'url': url, 'name': name})
                return u' | '.join(output)
            except Exception, e:
                return u''

        @monkeypatch_method(cls)
        def available_translations_admin(self, *args, **kwargs):
            """
            Gets a HTML with all available translation URLs for current object if available. For admin use.

            :return str:
            """
            return self._available_translations_admin(include_self=True, *args, **kwargs)
        available_translations_admin.allow_tags = True
        available_translations_admin.short_description = _('Translations')

        @monkeypatch_method(cls)
        def available_translations_exclude_current_admin(self, *args, **kwargs):
            """
            Same as `available_translations_admin` but does not include itself to the list.

            :return str:
            """
            return self._available_translations_admin(include_self=False, *args, **kwargs)
        available_translations_exclude_current_admin.allow_tags = True
        available_translations_exclude_current_admin.short_description = _('Translations')

        @monkeypatch_property(cls)
        def original_translation(self):
            """
            Property for ``get_original_translation`` method.

            :return obj: Object of the same class as the one queried.
            """
            return self.get_original_translation()

        @monkeypatch_method(cls)
        def get_translation_for(self, language):
            """
            Get translation article in given language.

            :param str language: Which shall be one of the languages specified in ``LANGUAGES``
                in `settings.py`.
            :return obj: Either object of the same class as or None if no translations are
                available for the given ``language``.
            """
            if not language in get_languages_keys():
                return None
            if str(self.language) == str(language):
                return self
            if str(self.original_translation.language) == str(language):
                return self.original_translation
            try:
                return self.original_translation.translations.get(language=language)
            except Exception, e:
                return None

    def formfield(self, **kwargs):
        """
        Returns best form field to represent this model field
        """
        defaults = {
            'form_class': LanguageField,
            'populate': self.populate,
        }
        defaults.update(kwargs)
        return super(models.CharField, self).formfield(**defaults)

    def validate(self, value, model_instance):
        """
        Validating the field.

        We shall make sure that there are double translations for the same language for the same object. That's
        why, in case if model is not yet saved (``translated_object`` does not yet have a primary key), we check
        if there are already translations of the same object in the language we specify now.

        Otherwise, if ``model_instance`` already has a primary key, we anyway try to get a ``translated_object``
        and compare it with our ``model_instance``. In case if ``translated_object`` exists and not equal to our
        ``model_instance`` we raise an error.

        NOTE: This has nothing to do with unique fields in the original ``model_instance``. Make sure you have
        properly specified all unique attributes with respect to ``LanguageField` of your original
        ``model_instance`` if you need those records to be unique.
        """
        if model_instance.original_translation:
            translated_object = model_instance.original_translation.get_translation_for(value)
        else:
            translated_object = None
        if not model_instance.pk:
            if translated_object and translated_object.pk:
                raise exceptions.ValidationError("Translation in language %s for this object already exists." % value )
        else:
            if translated_object and translated_object.pk and model_instance != translated_object:
                raise exceptions.ValidationError("Translation in language %s for this object already exists." % value )
        super(LanguageField, self).validate(value, model_instance)

class SimpleLanguageField(models.CharField):
    """
    SimpleLanguageField model. Stores language string in a ``CharField`` field.
    """

    def __init__(self, *args, **kwargs):
        """
        Create new field. Argument ``populate`` will be sent as-is to the form field.
        """
        defaults = {
            'verbose_name': _('Language'),
            'populate': None,
            'max_length': 10,
            'choices': get_languages(),
            'default': default_language
        }
        defaults.update(kwargs)
        self.populate = defaults.pop('populate', None)
        super(SimpleLanguageField, self).__init__(*args, **defaults)

    def formfield(self, **kwargs):
        """
        Returns best form field to represent this model field
        """
        defaults = {
            'form_class': SimpleLanguageField,
            'populate': self.populate,
        }
        defaults.update(kwargs)
        return super(models.CharField, self).formfield(**defaults)

# Add schema's for South
try:
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules(
        rules=[((LanguageField,), [], {'populate': ('populate', {'default':None}),})], \
        patterns=['slim.models\.fields\.LanguageField', 'slim.models\.fields\.SimpleLanguageField']
        )
except ImportError:
    pass



__all__ = ('locale_url_is_installed',)

from django.conf import settings

def locale_url_is_installed():
    """
    Checks if localeurl is installed in the Django project.

    :return bool:
    """
    if 'localeurl' in settings.INSTALLED_APPS and \
       'localeurl.middleware.LocaleURLMiddleware' in settings.MIDDLEWARE_CLASSES:
       return True
    return False


__title__ = 'slim.helpers'
__version__ = '0.1'
__build__ = 0x000001
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__all__ = ('get_default_language', 'default_language', 'get_languages', 'get_languages_keys', \
           'get_language_from_request', 'smart_resolve')

from django.conf import settings


def get_default_language():
    """
    Gets default language.

    :return str:
    """
    return settings.LANGUAGES[0][0]
default_language = get_default_language()


def get_languages():
    """
    Gets available languages.

    :return iterable:
    """
    return settings.LANGUAGES


def get_languages_keys():
    """
    Returns just languages keys.

    :return list:
    """
    return [key for key, name in get_languages()]


def get_language_from_request(request, default=default_language):
    """
    Gets language from HttpRequest

    :param django.http.HttpRequest:
    :param str default:
    :return str:
    """
    if hasattr(request, 'LANGUAGE_CODE') and request.LANGUAGE_CODE:
        return request.LANGUAGE_CODE
    else:
        return default


def smart_resolve(var, context):
    """
    Resolves variable from context in a smart way. First trying to resolve from context
    and when result is None checks if variable is not None and returns just variable
    when not. Otherwise returns None.

    :param str var:
    :param Context context:
    :return mixed:
    """
    if var is None:
        return None

    ret_val = None
    try:
        ret_val = var.resolve(context, True)
    except:
        ret_val = var
    if ret_val is None:
        ret_val = var

    return ret_val



__title__ = 'slim.templatetags.slim_tags'
__version__ = '0.1'
__build__ = 0x000001
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__all__ = ('get_translated_object_for', 'get_translated_objects_for', 'set_language', 'get_language_name_by_key', \
           'multiling_is_enabled')

import re

from django.utils.translation import ugettext_lazy as _
from django import template
from django.template import Library
from django.utils import translation

from slim.helpers import smart_resolve, default_language, get_language_from_request, get_languages_keys

register = Library()


class GetTranslatedObjectForNode(template.Node):
    """
    Node for template tag ``get_translated_object_for``.
    """
    def __init__(self, obj, as_var, language=None):
        """
        :param obj: Object to translate.
        :param string as_var: Desired variable name in the template context.
        :param str language: Language code. Must be one of those defined in ``LANGUAGES``
            of "settigns.py"
        :raise template.TemplateSyntaxError: If invalid syntax or in case if HttpRequest
            can't be retrieved from template context, while ``language`` attribute is not
            specified.
        """
        self.as_var = as_var
        self.obj = obj
        self.language = language

    def render(self, context):
        ret_val = None
        obj = smart_resolve(self.obj, context)

        try:
            is_multilingual = obj.is_multilingual
        except AttributeError, e:
            is_multilingual = False
            raise template.TemplateSyntaxError(
                _("Invalid usage of ``get_translated_object_for``. Translated object shall be multilingual.")
                )

        try:
            request = context['request']
        except:
            request = None

        if language is None and request is None:
            raise template.TemplateSyntaxError(
                _("Invalid usage of ``get_translated_object_for``. Can't retrieve ``HttpRequest`` object from "
                   "template context, while ``language`` attribute is not specified.")
                )

        if language is None:
            language = get_language_from_request(request)
        else:
            language = smart_resolve(self.language, context)
            if language:
                language = str(language)
            else:
                language = default_language

        ret_val = obj.get_translation_for(language)

        if self.as_var:
            context[self.as_var] = ret_val
            return ''
        return ret_val


@register.tag
def get_translated_object_for(parser, token):
    """
    Gets translated object for the object given.

    Syntax::
        {% get_translated_object_for [object] language=[language] as [var_name] %}

    Example usage::
        {% get_translated_object_for article as translated_article %}
        {% get_translated_object_for article language=ru as translated_article %}
    """
    bits = token.contents.split()
    if 'as' != bits[-2]:
        raise template.TemplateSyntaxError(
            _("Invalid syntax for %s. You must specify a name for translated object." % bits[0])
            )
    as_var = bits[-1]
    try:
        obj = parser.compile_filter(bits[1])
    except:
        template.TemplateSyntaxError(_("Invalid syntax for %s. You must provide an object to translate." % bits[0]))

    m = re.search(r'language=(\w+)', ' '.join(bits[2:-2]))

    if m:
        language = parser.compile_filter(m.groups()[0])
    else:
        language = None

    return GetTranslatedObjectForNode(obj=obj, as_var=as_var, language=language)


class GetTranslatedObjectsForNode(template.Node):
    """
    Node for template tag ``get_translated_objects_for``.
    """
    def __init__(self, obj, as_var):
        """
        :param obj: Object to get translations for.
        :param str as_var: Desired variable name in the template context.
        :raise template.TemplateSyntaxError: If invalid syntax or in case if HttpRequest
            can't be retrieved from template context, while ``language`` attribute is not
            specified.
        """
        self.as_var = as_var
        self.obj = obj

    def render(self, context):
        ret_val = None
        obj = smart_resolve(self.obj, context)

        try:
            is_multilingual = obj.is_multilingual
        except AttributeError, e:
            is_multilingual = False
            raise template.TemplateSyntaxError(
                _("Invalid usage of get_translated_object_for. Translated object shall be multilingual.")
                )

        ret_val = obj.available_translations()

        if self.as_var:
            context[self.as_var] = ret_val
            return ''
        return ret_val


@register.tag
def get_translated_objects_for(parser, token):
    """
    Gets translations available for the given object.

    Syntax::
        {% get_translated_objects_for [object] as [var_name] %}

    Example usage::
        {% get_translated_objects_for article as translated_article %}
    """
    bits = token.contents.split()
    if 'as' != bits[-2]:
        raise template.TemplateSyntaxError(
            _("Invalid syntax for %s. You must specify a name for translated object." % bits[0])
            )
    as_var = bits[-1]
    try:
        obj = parser.compile_filter(bits[1])
    except:
        template.TemplateSyntaxError(_("Invalid syntax for %s. You must provide an object to translate." % bits[0]))

    return GetTranslatedObjectsForNode(obj=obj, as_var=as_var)


class SetLanguageNode(template.Node):
    """
    Node for ``set_language`` tag.
    """
    def __init__(self, language=None):
        self.language = language

    def render(self, context):
        # Try to get request.LANGUAGE_CODE. If fail, use default one.
        request = context['request']
        language = get_language_from_request(request, default=None)

        if not language:
            language = smart_resolve(self.language, context)
            if not language in get_languages_keys():
                language = default_language

        translation.activate(language)
        return ''


@register.tag
def set_language(parser, token):
    """
    Sets current language code.

    FIXME: This is actually a hack.

    Syntax::
        {% set_language [language] %}
    Example::
        {% set_language ru %}
    """
    bits = token.contents.split()
    if 2 < len(bits):
        raise template.TemplateSyntaxError("'%s' tag takes one argument at most" % bits[0])
    elif 2 == len(bits):
        language = parser.compile_filter(bits[1])
    else:
        language = None
    return SetLanguageNode(language=language)


class GetLanguageNameByKeyNode(template.Node):
    """
    Node for ``get_language_name_by_key`` tag.
    """
    def __init__(self, key, as_var=None, translate=False):
        self.key = key
        self.as_var = as_var
        self.translate = translate

    def render(self, context):
        # Try to get request.LANGUAGE_CODE. If fail, use default one.
        key = smart_resolve(self.key, context)
        translate = smart_resolve(self.translate, context)
        languages = dict(get_languages())

        if languages.has_key(key):
            ret_val = languages[key]
        else:
            ret_val = None

        if translate:
            ret_val = ugettext(ret_val)

        if self.as_var:
            context[self.as_var] = ret_val
            return ''
        return ret_val


@register.tag
def get_language_name_by_key(parser, token):
    """
    Gets language name by key given

    FIXME: This is actually a hack.

    Syntax::
        {% get_language_name_by_key [key] [translate] as [var_name] %}
    Example::
        {% get_language_name_by_key ru translate as language_key %}
    """
    bits = token.contents.split()
    if 5 < len(bits):
        raise template.TemplateSyntaxError("'%s' tag takes five arguments at most" % bits[0])

    key = parser.compile_filter(bits[1])

    m = re.search(r'translate=(\d)', ' '.join(bits[2:-2]))

    if m:
        translate = parser.compile_filter(m.groups()[0])
    else:
        translate = None

    if 'as' != bits[-2]:
        raise template.TemplateSyntaxError(
            _("Invalid syntax for %s. You must specify a name for translated object." % bits[0])
            )
    as_var = bits[-1]

    return GetLanguageNameByKeyNode(key=key, as_var=as_var, translate=translate)


class MultilinIsEnabledNode(template.Node):
    """
    Node for ``multiling_is_enabled`` tag.
    """
    def __init__(self, as_var=None):
        self.as_var = as_var

    def render(self, context):
        # Try to get request.LANGUAGE_CODE. If fail, use default one.
        if len(get_languages_keys()) > 1:
            ret_val = True
        else:
            ret_val = False

        if self.as_var:
            context[self.as_var] = ret_val
            return ''
        return ret_val


@register.tag
def multiling_is_enabled(parser, token):
    """
    Checks if multiling shall be enabled (in templates). Simply, if LANGUAGES tuple contains more than one
    language, we return boolean True; otherwise - boolean False.

    Syntax::
        {% multiling_is_enabled as [var_name] %}
    Example::
        {% multiling_is_enabled as multiling_is_enabled %}
    """
    bits = token.contents.split()
    if not len(bits) in (1, 3):
        raise template.TemplateSyntaxError("'%s' tag takes five arguments at most" % bits[0])

    if 3 == len(bits):
        if 'as' != bits[-2]:
            raise template.TemplateSyntaxError(
                _("Invalid syntax for %s. You must specify a name for translated object." % bits[0])
                )
        as_var = bits[-1]
    else:
        as_var = None

    return MultilinIsEnabledNode(as_var=as_var)



import os
from setuptools import setup, find_packages

try:
  readme = open(os.path.join(os.path.dirname(__file__), 'readme.rst')).read()
except:
  readme = ''

version = '0.2'

setup(
    name='django-slim',
    version=version,
    description=("Generate a random date from range given."),
    long_description=readme,
    classifiers=[
        "Programming Language :: Python",
        "Environment :: Web Environment",
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
    ],
    keywords='multi-lingual, django, python',
    author='Artur Barseghyan',
    author_email='artur.barseghyan@gmail.com',
    package_dir={'':'src'},
    packages=find_packages(where='./src'),
    url='https://bitbucket.org/barseghyanartur/django-slim',
    license='GPL 2.0/LGPL 2.1',
    #install_requires=['python-dateutil']
)



__all__ = ('get_setting',)

from django.conf import settings

from slim import defaults

def get_setting(setting, override=None):
    """
    Get a setting from ``slim`` conf module, falling back to the default.

    If override is not None, it will be used instead of the setting.
    """
    if override is not None:
        return override
    if hasattr(settings, 'SLIM_%s' % setting):
        return getattr(settings, 'SLIM_%s' % setting)
    else:
        return getattr(defaults, setting)